/* ----------------------------------------------------------------------------
 *
 * Raima Database Manager
 *
 * Copyright (c) 2012 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 *
 * ----------------------------------------------------------------------------
 */

#include <stdio.h>

/* rdm.h is required in all files using the RDM Database */
#include "rdm.h"

/* core05_db.h was generated from the DDL file core05_db.ddl by the ddlp.
 * utility.
 */
#include "core05_db.h"

/* core05_db_dbd.h was generated from the DDL file core05_db.ddl by the 
 * ddlp utility.  It is required to use the d_open_ptr API
 */
#include "core05_db_dbd.h"

/* EXAMPLE - Core for C */
/* Each time you run this example:
 *   1) The database is initialized (if necessary)
 *   2) 5 new person records are added to the database
 *   3) A list of each person is displayed in index order.
 *
 * The DDL (Database Definition Language) is in core05_db.ddl. The database
 * schema is embedded into the application and the database will be created 
 * the first time the application is run.
 *
 * The following RDM Functions are illustrated:
 *      d_opentask/d_closetask
 *      d_open_ptr/d_close
 *      d_initialize
 *      d_fillnew
 *      d_keyfrst/d_keynext
 *      d_recread
 *      d_errorname
 *      d_errorstr
 *
 * They are fully documented in http://docs.raima.com/rdm/.
 *
 * Because the database is opened in exclusive mode lock and transactions
 * are not required.  Future core examples will use shared mode and will
 * demonstrate the use of transactions and locks.
 *
 * Each of these functions returns an integer status code, where the
 * value S_OKAY indicates a successful call.
 *
 * For simplicity, this example does not check all return codes, but good
 * programming practices would dictate that they are checked after each
 * RDM call.
 *
 * The actual database will be stored in a directory named 'core05_db' in the
 * project directory.
 */

/* =======================================================================
*/
int32_t main(void)
{
    DB_TASK       *task;
    int32_t        status;
    int32_t        value;
    uint16_t       ii;
    struct person  person_rec;

    /* Seed the random number generator */
    srand(time(NULL));

    /* create the context for the database operations */
    d_opentask(&task);

    /* open the database for exclusive access, this will create the database
     * dictionary (core05_db.dbd) file if it does not already exist.
     */
    status = d_open_ptr("core05_db", "x", &core05_db_dbd, sizeof(core05_db_dbd), task);
    if (status != S_OKAY)
    {
        fprintf (stderr, "Exclusive open failed, error = %s: %s\n", d_errorname(status), d_errorstr(status) );
        d_closetask (task);
        return (1);
    }

    /* Check to see if the database data file already exists, if not d_recfrst
     * will return S_NOFILE and we know we need to initialize the database.
     * Initializing the database will create new data/key files..
     */
    status = d_recfrst(PERSON, task, CURR_DB);
    if(status == S_NOFILE)
    {
        d_initialize(task, CURR_DB);
    }

    /* add 5 new records to the database */
    printf("Adding 5 new person records\n");
    for (ii = 0; ii < 5; ii++)
    {
        value = rand() % 10000;
        sprintf(person_rec.last_four_ssn, "%04d", value);
        d_fillnew(PERSON, &person_rec, task, CURR_DB);

        printf("\t%s\n", person_rec.last_four_ssn);
    }

    /* Loop through and display each person record through the index
     */
    printf("\nList all records in index order\n");
    for (status = d_keyfrst (PERSON_LAST_FOUR_SSN, task, CURR_DB);
         status == S_OKAY;
         status = d_keynext (PERSON_LAST_FOUR_SSN, task, CURR_DB))
    {
        /* Read and display the current person record */
        d_recread(&person_rec, task, CURR_DB);
        printf("%s\n", person_rec.last_four_ssn);
    }
    
    /* close the database */
    d_close (task);
    d_closetask (task);
    
    printf("\nPress <Enter> to continue . . .\n");
    getchar();

    return EXIT_SUCCESS;
}
